@@ -72,6 +72,7 @@ def run_bootstrap_opencl(best_theta, G_m_buf, indices_array_np, k, min_energy, i
     queue = opencl_context.queue
     bootstrap_kernel = opencl_context.bootstrap_segmented_kernel if is_segmented else opencl_context.bootstrap_kernel
     dtype = opencl_context.dtype
+    epsilon = opencl_context.epsilon
     wgs = opencl_context.work_group_size
 
     n = best_theta.shape[0]
@@ -143,7 +144,7 @@ def run_bootstrap_opencl(best_theta, G_m_buf, indices_array_np, k, min_energy, i
     if min_energy < energy:
         return min_energy
 
-    atol = dtype(1.1920928955078125e-7)
+    atol = dtype(epsilon)
     rtol = dtype(0)
     choices = np.where(np.isclose(min_energy_host, energy, atol=atol, rtol=rtol))[0]
     best_i = np.random.choice(choices) if len(choices) else np.argmin(min_energy_host)
@@ -162,9 +163,10 @@ def spin_glass_solver(
     quality=None,
     shots=None,
     best_guess=None,
-    is_alt_gpu_sampling=False,
-    is_base_maxcut_gpu=True,
-    is_combo_maxcut_gpu=True
+    is_combo_maxcut_gpu=True,
+    is_spin_glass=True,
+    anneal_t=None,
+    anneal_h=None
 ):
     dtype = opencl_context.dtype
     nodes = None
@@ -199,9 +201,6 @@ def spin_glass_solver(
 
             return "01", weight, ([nodes[0]], [nodes[1]]), -weight
 
-    if quality is None:
-        quality = 3
-
     bitstring = ""
     if isinstance(best_guess, str):
         bitstring = best_guess
@@ -210,7 +209,7 @@ def spin_glass_solver(
     elif isinstance(best_guess, list):
         bitstring = "".join(["1" if b else "0" for b in best_guess])
     else:
-        bitstring, _, _ = maxcut_tfim(G_m, quality=quality, shots=shots, is_alt_gpu_sampling=is_alt_gpu_sampling, is_g_buf_reused=True, is_base_maxcut_gpu=is_base_maxcut_gpu)
+        bitstring, _, _ = maxcut_tfim(G_m, quality=quality, shots=shots, is_spin_glass=is_spin_glass, anneal_t=anneal_t, anneal_h=anneal_h)
     best_theta = np.array([b == "1" for b in list(bitstring)], dtype=np.bool_)
 
     if is_combo_maxcut_gpu and IS_OPENCL_AVAILABLE:
