@@ -75,6 +75,7 @@ def run_bootstrap_opencl(best_theta, G_data_buf, G_rows_buf, G_cols_buf, indices
     queue = opencl_context.queue
     bootstrap_kernel = opencl_context.bootstrap_sparse_segmented_kernel if is_segmented else opencl_context.bootstrap_sparse_kernel
     dtype = opencl_context.dtype
+    epsilon = opencl_context.epsilon
     wgs = opencl_context.work_group_size
 
     n = best_theta.shape[0]
@@ -150,7 +151,7 @@ def run_bootstrap_opencl(best_theta, G_data_buf, G_rows_buf, G_cols_buf, indices
     if min_energy < energy:
         return min_energy
 
-    atol = dtype(1.1920928955078125e-7)
+    atol = dtype(epsilon)
     rtol = dtype(0)
     choices = np.where(np.isclose(min_energy_host, energy, atol=atol, rtol=rtol))[0]
     best_i = np.random.choice(choices) if len(choices) else np.argmin(min_energy_host)
@@ -181,9 +182,10 @@ def spin_glass_solver_sparse(
     quality=None,
     shots=None,
     best_guess=None,
-    is_alt_gpu_sampling=False,
-    is_base_maxcut_gpu=True,
-    is_combo_maxcut_gpu=True
+    is_combo_maxcut_gpu=True,
+    is_spin_glass=True,
+    anneal_t=None,
+    anneal_h=None
 ):
     dtype = opencl_context.dtype
     nodes = None
@@ -192,7 +194,7 @@ def spin_glass_solver_sparse(
     if isinstance(G, nx.Graph):
         nodes = list(G.nodes())
         n_qubits = len(nodes)
-        G_m = to_scipy_sparse_upper_triangular(G, nodes, n_qubits)
+        G_m = to_scipy_sparse_upper_triangular(G, nodes, n_qubits, dtype)
     else:
         n_qubits = G.shape[0]
         nodes = list(range(n_qubits))
@@ -218,9 +220,6 @@ def spin_glass_solver_sparse(
         print("[WARN] Using segmented solver, so disabling is_alt_gpu_sampling.")
         is_alt_gpu_sampling = False
 
-    if quality is None:
-        quuality = 3
-
     bitstring = ""
     if isinstance(best_guess, str):
         bitstring = best_guess
@@ -229,7 +228,7 @@ def spin_glass_solver_sparse(
     elif isinstance(best_guess, list):
         bitstring = "".join(["1" if b else "0" for b in best_guess])
     else:
-        bitstring, _, _ = maxcut_tfim_sparse(G_m, quality=quality, shots=shots, is_alt_gpu_sampling=is_alt_gpu_sampling, is_g_buf_reused=True, is_base_maxcut_gpu=is_base_maxcut_gpu)
+        bitstring, _, _ = maxcut_tfim_sparse(G_m, quality=quality, shots=shots, is_spin_glass=is_spin_glass, anneal_t=anneal_t, anneal_h=anneal_h)
     best_theta = np.array([b == "1" for b in list(bitstring)], dtype=np.bool_)
 
     if is_combo_maxcut_gpu and IS_OPENCL_AVAILABLE:
