@@ -8,6 +8,9 @@ from numba import njit, prange
 import os
 
 
+epsilon = opencl_context.epsilon
+
+
 @njit
 def evaluate_cut_edges(theta_bits, G_func, nodes):
     n_qubits = len(nodes)
@@ -52,7 +55,7 @@ def bootstrap(best_theta, G_func, nodes, indices_array, k, min_energy, dtype):
 
     energy = energies.min()
     if energy < min_energy:
-        atol = dtype(1.1920928955078125e-7)
+        atol = dtype(epsilon)
         rtol = dtype(0)
         index_match = np.random.choice(np.where(np.isclose(energies, energy, atol=atol, rtol=rtol))[0])
         indices = indices_array[(index_match * k) : ((index_match + 1) * k)]
@@ -69,7 +72,9 @@ def spin_glass_solver_streaming(
     quality=None,
     shots=None,
     best_guess=None,
-    is_base_maxcut_gpu=True
+    is_spin_glass=True,
+    anneal_t=None,
+    anneal_h=None
 ):
     dtype = opencl_context.dtype
     n_qubits = len(nodes)
@@ -88,9 +93,6 @@ def spin_glass_solver_streaming(
 
             return "01", weight, ([nodes[0]], [nodes[1]]), -weight
 
-    if quality is None:
-        quality = 3
-
     bitstring = ""
     if isinstance(best_guess, str):
         bitstring = best_guess
@@ -99,7 +101,7 @@ def spin_glass_solver_streaming(
     elif isinstance(best_guess, list):
         bitstring = "".join(["1" if b else "0" for b in best_guess])
     else:
-        bitstring, _, _ = maxcut_tfim_streaming(G_func, nodes, quality=quality, shots=shots, is_base_maxcut_gpu=is_base_maxcut_gpu)
+        bitstring, _, _ = maxcut_tfim_streaming(G_func, nodes, quality=quality, shots=shots, is_spin_glass=is_spin_glass, anneal_t=anneal_t, anneal_h=anneal_h)
     best_theta = np.array([b == "1" for b in list(bitstring)], dtype=np.bool_)
 
     min_energy = compute_energy(best_theta, G_func, nodes)
