@@ -1,5 +1,5 @@
-from .spin_glass_solver import spin_glass_solver
 from .maxcut_tfim_util import binary_search, opencl_context, to_scipy_sparse_upper_triangular
+from .maxcut_tfim import maxcut_tfim
 from concurrent.futures import ProcessPoolExecutor
 import time
 import itertools
@@ -283,9 +283,9 @@ def targeted_three_opt_parallel(path, W, neighbor_lists, k_neighbors=20):
                         dists[l] = path_length(new_path, W)
                         paths[l] = new_path
 
-                    for i in range(7):
-                        if dists[i] < best_dist:
-                            best_path, best_dist, improved = paths[i], dists[i], True
+                    for l in range(7):
+                        if dists[l] < best_dist:
+                            best_path, best_dist, improved = paths[l], dists[l], True
 
                     if improved:
                         break
@@ -627,9 +627,8 @@ def tsp_symmetric(
     end_node=None,
     quality=None,
     shots=None,
-    is_alt_gpu_sampling=False,
-    is_base_maxcut_gpu=True,
-    is_combo_maxcut_gpu=True,
+    anneal_t=None,
+    anneal_h=None,
     monte_carlo=True,
     k_neighbors=20,
     is_cyclic=True,
@@ -677,13 +676,13 @@ def tsp_symmetric(
         if monte_carlo:
             a, b = monte_carlo_loop(n_nodes)
         else:
-            best_energy = float("inf")
+            best_cut = -float("inf")
             for _ in range(multi_start):
                 bits = ([], [])
                 while (len(bits[0]) == 0) or (len(bits[1]) == 0):
-                    _, _, bits, energy = spin_glass_solver(G_m, quality=quality, shots=shots, is_alt_gpu_sampling=is_alt_gpu_sampling, is_base_maxcut_gpu=is_base_maxcut_gpu, is_combo_maxcut_gpu=is_combo_maxcut_gpu)
-                if energy < best_energy:
-                    best_energy = energy
+                    _, cut_value, bits = maxcut_tfim(G_m, quality=quality, shots=shots, anneal_t=anneal_t, anneal_h=anneal_h)
+                if cut_value > best_cut:
+                    best_cut = cut_value
                     a, b = bits
     else:
         is_cyclic = False
@@ -698,17 +697,16 @@ def tsp_symmetric(
 
     if monte_carlo and is_parallel and (parallel_level < max_parallel_level) and (len(a) > 3) and (len(b) > 3):
         with ProcessPoolExecutor(max_workers=1) as executor:
-            f = executor.submit(tsp_symmetric, G_a, None, None, quality, shots, is_alt_gpu_sampling, is_base_maxcut_gpu, is_combo_maxcut_gpu, True, 0, False, multi_start, False, True, parallel_level + 1)
-            sol_b = tsp_symmetric(G_b, None, None, quality, shots, is_alt_gpu_sampling, is_base_maxcut_gpu, is_combo_maxcut_gpu, True, 0, False, multi_start, False, True, parallel_level + 1)
+            f = executor.submit(tsp_symmetric, G_a, None, None, quality, shots, anneal_t, anneal_h, True, 0, False, multi_start, False, True, parallel_level + 1)
+            sol_b = tsp_symmetric(G_b, None, None, quality, shots, anneal_t, anneal_h, True, 0, False, multi_start, False, True, parallel_level + 1)
             sol_a = f.result()
     else:
         sol_a = tsp_symmetric(
             G_a,
             quality=quality,
             shots=shots,
-            is_alt_gpu_sampling=is_alt_gpu_sampling,
-            is_base_maxcut_gpu=is_base_maxcut_gpu,
-            is_combo_maxcut_gpu=is_combo_maxcut_gpu,
+            anneal_t=anneal_t,
+            anneal_h=anneal_h,
             monte_carlo=monte_carlo,
             is_cyclic=False,
             is_top_level=False,
@@ -720,9 +718,8 @@ def tsp_symmetric(
             G_b,
             quality=quality,
             shots=shots,
-            is_alt_gpu_sampling=is_alt_gpu_sampling,
-            is_base_maxcut_gpu=is_base_maxcut_gpu,
-            is_combo_maxcut_gpu=is_combo_maxcut_gpu,
+            anneal_t=anneal_t,
+            anneal_h=anneal_h,
             monte_carlo=monte_carlo,
             is_cyclic=False,
             is_top_level=False,
@@ -833,9 +830,8 @@ def tsp_asymmetric(
     end_node=None,
     quality=None,
     shots=None,
-    is_alt_gpu_sampling=False,
-    is_base_maxcut_gpu=True,
-    is_combo_maxcut_gpu=True,
+    anneal_t=None,
+    anneal_h=None,
     monte_carlo=True,
     k_neighbors=20,
     is_cyclic=True,
@@ -887,13 +883,13 @@ def tsp_asymmetric(
         if monte_carlo:
             a, b = monte_carlo_loop(n_nodes)
         else:
-            best_energy = float("inf")
+            best_cut = -float("inf")
             for _ in range(multi_start):
                 bits = ([], [])
                 while (len(bits[0]) == 0) or (len(bits[1]) == 0):
-                    _, _, bits, energy = spin_glass_solver((G_m + G_m.T) / 2, quality=quality, shots=shots, is_alt_gpu_sampling=is_alt_gpu_sampling, is_base_maxcut_gpu=is_base_maxcut_gpu, is_combo_maxcut_gpu=is_combo_maxcut_gpu)
-                if energy < best_energy:
-                    best_energy = energy
+                    _, cut_value, bits = maxcut_tfim((G_m + G_m.T) / 2, quality=quality, shots=shots, anneal_t=anneal_t, anneal_h=anneal_h)
+                if cut_value > best_cut:
+                    best_cut = cut_value
                     a, b = bits
     else:
         is_cyclic = False
@@ -908,17 +904,16 @@ def tsp_asymmetric(
 
     if monte_carlo and is_parallel and (parallel_level < max_parallel_level) and (len(a) > 2) and (len(b) > 2):
         with ProcessPoolExecutor(max_workers=1) as executor:
-            f = executor.submit(tsp_asymmetric, G_a, None, None, quality, shots, is_alt_gpu_sampling, is_base_maxcut_gpu, is_combo_maxcut_gpu, True, 0, False, multi_start, False, True, parallel_level + 1)
-            sol_b = tsp_asymmetric(G_b, None, None, quality, shots, is_alt_gpu_sampling, is_base_maxcut_gpu, is_combo_maxcut_gpu, True, 0, False, multi_start, False, True, parallel_level + 1)
+            f = executor.submit(tsp_asymmetric, G_a, None, None, quality, shots, anneal_t, anneal_h, True, 0, False, multi_start, False, True, parallel_level + 1)
+            sol_b = tsp_asymmetric(G_b, None, None, quality, shots, anneal_t, anneal_h, True, 0, False, multi_start, False, True, parallel_level + 1)
             sol_a = f.result()
     else:
         sol_a = tsp_asymmetric(
             G_a,
             quality=quality,
             shots=shots,
-            is_alt_gpu_sampling=is_alt_gpu_sampling,
-            is_base_maxcut_gpu=is_base_maxcut_gpu,
-            is_combo_maxcut_gpu=is_combo_maxcut_gpu,
+            anneal_t=anneal_t,
+            anneal_h=anneal_h,
             monte_carlo=monte_carlo,
             is_cyclic=False,
             is_top_level=False,
@@ -930,9 +925,8 @@ def tsp_asymmetric(
             G_b,
             quality=quality,
             shots=shots,
-            is_alt_gpu_sampling=is_alt_gpu_sampling,
-            is_base_maxcut_gpu=is_base_maxcut_gpu,
-            is_combo_maxcut_gpu=is_combo_maxcut_gpu,
+            anneal_t=anneal_t,
+            anneal_h=anneal_h,
             monte_carlo=monte_carlo,
             is_cyclic=False,
             is_top_level=False,
@@ -1155,9 +1149,9 @@ def targeted_three_opt_sparse_parallel(path, G_data, G_rows, G_cols, neighbor_li
                         dists[l] = path_length_sparse(new_path, G_data, G_rows, G_cols)
                         paths[l] = new_path
 
-                    for i in range(7):
-                        if dists[i] < best_dist:
-                            best_path, best_dist, improved = paths[i], dists[i], True
+                    for l in range(7):
+                        if dists[l] < best_dist:
+                            best_path, best_dist, improved = paths[l], dists[l], True
 
                     if improved:
                         break
@@ -1389,8 +1383,8 @@ def tsp_symmetric_sparse(
 
     if is_parallel and (parallel_level < max_parallel_level) and (len(a) > 3) and (len(b) > 3):
         with ProcessPoolExecutor(max_workers=1) as executor:
-            f = executor.submit(tsp_symmetric_sparse, G_a, False, True, parallel_level + 1)
-            sol_b = tsp_symmetric_sparse(G_b, False, True, parallel_level + 1)
+            f = executor.submit(tsp_symmetric_sparse, G_a, 0, False, True, parallel_level + 1)
+            sol_b = tsp_symmetric_sparse(G_b, 0, False, True, parallel_level + 1)
             sol_a = f.result()
     else:
         sol_a = tsp_symmetric_sparse(
@@ -1595,9 +1589,9 @@ def targeted_three_opt_streaming_parallel(path, G_func, neighbor_lists, k_neighb
                         dists[l] = path_length_streaming(new_path, G_func)
                         paths[l] = new_path
 
-                    for i in range(7):
-                        if dists[i] < best_dist:
-                            best_path, best_dist, improved = paths[i], dists[i], True
+                    for l in range(7):
+                        if dists[l] < best_dist:
+                            best_path, best_dist, improved = paths[l], dists[l], True
 
                     if improved:
                         break
@@ -1782,8 +1776,8 @@ def tsp_symmetric_streaming(
 
     if is_parallel and (parallel_level < max_parallel_level) and (len(a) > 3) and (len(b) > 3):
         with ProcessPoolExecutor(max_workers=1) as executor:
-            f = executor.submit(tsp_symmetric_streaming, G_func, a, False, True, parallel_level + 1)
-            sol_b = tsp_symmetric_streaming(G_func, b, False, True, parallel_level + 1)
+            f = executor.submit(tsp_symmetric_streaming, G_func, a, 0, False, True, parallel_level + 1)
+            sol_b = tsp_symmetric_streaming(G_func, b, 0, False, True, parallel_level + 1)
             sol_a = f.result()
     else:
         sol_a = tsp_symmetric_streaming(
